﻿using System;

namespace UMK.Barista
{
    //component
    public abstract class Napój
    {
        public abstract decimal Cena();
        public void Objętość() //funckja nie podlegająca zmianom podczas dekoracji
        {
            Console.WriteLine("\nObjetosc: 330 ml");
        }
    }

    //concrete components
    class Herbata : Napój
    {
        public override decimal Cena()
        {
            return 6.70M;
        }

        public override string ToString()
        {
            return "Herbata";
        }
    }

    class Kawa : Napój
    {
        public override decimal Cena()
        {
            return 7.90M;
        }

        public override string ToString()
        {
            return "Kawa";
        }
    }

    //dekorator
    abstract class NapójZDodatkiem : Napój
    {
        protected Napój napój;

        public NapójZDodatkiem(Napój napój)
        {
            this.napój = napój;
        }
    }

    //concrete decorator
    class NapójZeSzczyptąCynamonu : NapójZDodatkiem
    {
        public NapójZeSzczyptąCynamonu(Napój napój)
            :base(napój)
        { }

        public override decimal Cena()
        {
            return napój.Cena() + 0.5M;
        }

        public override string ToString()
        {
            return napój.ToString() + " ze szczyptą cynamonu";
        }
    }

    class NapójZCytryną : NapójZDodatkiem
    {
        private int ilePlasterków;

        public NapójZCytryną(Napój napój, int ilePlasterków)
            : base(napój)
        {
            this.ilePlasterków = ilePlasterków;
        }

        public override decimal Cena()
        {
            return napój.Cena() + ilePlasterków * 1M;
        }

        public override string ToString()
        {
            return napój.ToString() + $" z {ilePlasterków} plasterkami cytryny";
        }
    }

    class NapójZŁyżeczkąCukru : NapójZDodatkiem
    {
        public NapójZŁyżeczkąCukru(Napój napój)
            : base(napój)
        { }

        public override decimal Cena()
        {
            return napój.Cena() + 1.5M;
        }

        public override string ToString()
        {
            return napój.ToString() + " z łyżeczką cukru";
        }
    }
}
